function [ optimization_process_information ] = gbo( optimization_problem )
    assert(strcmp(optimization_problem.algorithm_c,'gbo'));
    
    optimization_process_information.x_opt_history = [];
    optimization_process_information.y_opt_history = [];
    optimization_process_information.alpha_history = [];
    optimization_process_information.alpha_pred_history = [];
    optimization_process_information.alpha_quotient = [];
    optimization_process_information.alpha_request_history = [];
    optimization_process_information.dLOO_history = [];
    optimization_process_information.dLOO_request_history = [];
    optimization_process_information.l_f_history = [];
    optimization_process_information.X = [];
    optimization_process_information.y = [];
    optimization_process_information.GP = cell(0);

    
    %% animation
    debug_animation_b = 1;
    
    %% create gaussian process
    GP = GP_interface(...
        optimization_problem.algorithm_parameters_t.covariance_c,...
        optimization_problem.algorithm_parameters_t.mean_c);
    GP.sensor.sigmaS_f = optimization_problem.sigmaS_f;
    
    %% configure covariance function (initial config only)
    GP.covariance.kernel_t.parameters_t.sigmaF_f = 1;
    GP.covariance.kernel_t.parameters_t.l_f = ...
        lowerBound( optimization_problem.algorithm_parameters_t.c_min,optimization_problem.dim_i, 1);
    if optimization_problem.algorithm_parameters_t.kernel_parameters_preset_b
        GP.covariance.kernel_t.parameters_t = ...
            optimization_problem.algorithm_parameters_t.kernel_t.parameters_t;
    end
    if optimization_problem.algorithm_parameters_t.mean_parameters_preset_b
        GP.mean.mean_t.parameters_t = ...
            optimization_problem.algorithm_parameters_t.mean_t.parameters_t;
    end
    
    %% create local minimum region detection
    [ local_min_regions_t ] = LOCALMINIMUMREGIONS_interface( 1000 );
    local_min_regions_t.search.type_c = 'quadReg';
    local_min_regions_t.search.parameters_t(1) = 10; % grid resolution
    local_min_regions_t.search.parameters_t(2) = 0.000001; % lambda
    local_min_regions_t.search.parameters_t(3) = 1; % search based on data
    local_min_regions_out_t = local_min_regions_t;

    %% get initial data via latin hypercube sampling
    optimization_problem.X_t = ...
        lhsdesign(optimization_problem.algorithm_parameters_t.numInitSamples_i,...
            optimization_problem.dim_i);
    optimization_problem.y_t = optimization_problem.fun_func(...
         transformInputSpace(optimization_problem.X_t,optimization_problem))';
    optimization_problem.n_data_i = numel(optimization_problem.y_t(:));
    
    %% get initial sub-optimization starting points (for optimization on surface)
    x0 = lhsdesign(optimization_problem.algorithm_parameters_t.N_opt_restarts_i,...
            optimization_problem.dim_i);
        
    %% main optimization loop
    for n = 1:1:optimization_problem.algorithm_parameters_t.N_i
        %% policy search
        % if any local optimum is detected check for policy value
        if optimization_problem.algorithm_parameters_t.direct_local_optimum_b ...
                && (local_min_regions_out_t.nSegments > 0)
            % policy search without local data (on SE kernel)
            [optimization_problem_reduced, GP_reduced] = removeIDFromData(...
                local_min_regions_out_t.segments_t(1).X_IDs, GP,...
                optimization_problem);
            if ~(optimization_problem_reduced.n_data_i < optimization_problem.algorithm_parameters_t.numInitSamples_i)
                f = createPolicyFcnHandle(GP_reduced, optimization_problem_reduced);
                [x_next, alpha] = kimOpt(optimization_problem_reduced,f,x0);
            else
                alpha = inf;
            end

            % calculate policy value in best local minimum region
            alpha_quad = plainPolicy(...
                local_min_regions_out_t.segments_t(1).objective,...
                0,...
                min(optimization_problem.y_t),...
                optimization_problem.dim_i,...
                optimization_problem.n_data_i,...
                optimization_problem.algorithm_parameters_t.policy_c);
            
            % decide if (best) local region wins
            if alpha_quad < alpha
                x_next = local_min_regions_out_t.segments_t(1).local_optimum;
                alpha = optimization_process_information.alpha_history(end);
            end
        else
            % ordinary policy search without local regions
            f = createPolicyFcnHandle(GP, optimization_problem);
            [x_next, alpha] = kimOpt(optimization_problem,f,x0);
        end
        
        % for logging
        optimization_process_information.alpha_history = ...
                [optimization_process_information.alpha_history;alpha];
            
        %% plot current gaussian process
        if debug_animation_b && n > 3 && optimization_problem.dim_i < 3
            figure(1);
            if (optimization_problem.dim_i ==1 && ( (~exist('local_min_regions_out_t')) || (exist('local_min_regions_out_t') && GP.mean.mean_t.parameters_t.local_min_regions_t.nSegments == 0)))
                N = 200; % number of samples
                x_ = linspace(0,1,N)';
                [ mean,sigma, K_cond ] = posterior( optimization_problem.X_t,...
                    x_,optimization_problem.y_t,GP );
                hold off;
                plot1DGaussian(optimization_problem.X_t,optimization_problem.y_t,...
                    x_,mean,K_cond);
                %scatter(optimization_problem.X_t(end,1),optimization_problem.X_t(end,2),optimization_problem.y_t(end),'c');
            end 
            if (optimization_problem.dim_i ==1 && exist('local_min_regions_out_t') && GP.mean.mean_t.parameters_t.local_min_regions_t.nSegments ~= 0 )
                N = 200; % number of samples
                x_ = linspace(0,1,N)';
                [ mean,sigma, K_cond ] = posterior( optimization_problem_reduced.X_t,...
                    x_,optimization_problem_reduced.y_t,GP_reduced );
                hold off;
                plot1DGaussian(optimization_problem_reduced.X_t,optimization_problem_reduced.y_t,...
                    x_,mean,K_cond);
                %scatter(optimization_problem.X_t(end,1),optimization_problem.X_t(end,2),optimization_problem.y_t(end),'c');
            end
            if (optimization_problem.dim_i ==2 && ( (~exist('local_min_regions_out_t')) || (exist('local_min_regions_out_t') && GP.mean.mean_t.parameters_t.local_min_regions_t.nSegments == 0)))
                N = 20; % number of samples
                [X,Y] = meshgrid(linspace(0,1,N),linspace(0,1,N));
                [ mean,sigma, K_cond ] = posterior( optimization_problem.X_t,...
                    [X(:),Y(:)],optimization_problem.y_t,GP );
                plot2DGaussian(optimization_problem.X_t,optimization_problem.y_t,...
                    X,Y, mean,K_cond);
            end 
            if (optimization_problem.dim_i ==2 && exist('local_min_regions_out_t') && GP.mean.mean_t.parameters_t.local_min_regions_t.nSegments ~= 0)
                N = 20; % number of samples
                [X,Y] = meshgrid(linspace(0,1,N),linspace(0,1,N));
                [ mean,sigma, K_cond ] = posterior( optimization_problem_reduced.X_t,...
                    [X(:),Y(:)],optimization_problem_reduced.y_t,GP_reduced );
                plot2DGaussian(optimization_problem_reduced.X_t,optimization_problem_reduced.y_t,...
                    X,Y, mean,K_cond);
            end 
            hold all
            if exist('local_min_regions_out_t')
                if (optimization_problem.dim_i <= 2 ) && GP.mean.mean_t.parameters_t.local_min_regions_t.nSegments ~= 0
                    visualizeAllMinimaRegions(local_min_regions_out_t,...
                        optimization_problem,1);
                end
            end
            plotConfiguration;
        end
        
        %% add data point via objective evaluation
        optimization_problem.X_t = [optimization_problem.X_t;...
            x_next];
        optimization_problem.y_t = [optimization_problem.y_t...
            ;optimization_problem.fun_func(...
            transformInputSpace(optimization_problem.X_t(end,:),optimization_problem))];
        optimization_problem.n_data_i = optimization_problem.n_data_i + 1;
        % show next evaluation point
        if (optimization_problem.dim_i == 1 && debug_animation_b == 1 && n>3)
            hold on
            scatter(optimization_problem.X_t(end,:),optimization_problem.y_t(end),'c');
            hold off
        end
        if (optimization_problem.dim_i == 2 && debug_animation_b == 1 && n>3)
            hold on
            scatter3(optimization_problem.X_t(end,1),optimization_problem.X_t(end,2),optimization_problem.y_t(end),'c');
            hold off
        end

        %% update model parameters
        GP = doStatisticalParameterEstimation(GP, optimization_problem);
                
        % update length-scale
        if optimization_problem.algorithm_parameters_t.lengthscaleControllerAlpha == 1
            if isempty(optimization_process_information.alpha_request_history)
                [GP, optimization_process_information] = lengthscaleControllerAlpha(...
                    GP, optimization_problem, optimization_process_information,x0);
            elseif ~optimization_process_information.alpha_request_history(end)==1
                [GP, optimization_process_information] = lengthscaleControllerAlpha(...
                    GP, optimization_problem, optimization_process_information,x0);
            else %mainly for analysis reasons...
                optimization_process_information.alpha_request_history(end+1) = 0;
                optimization_process_information.alpha_pred_history(end+1) = ...
                    optimization_process_information.alpha_pred_history(end);
                optimization_process_information.alpha_quotient(end+1) = ...
                    optimization_process_information.alpha_quotient(end);
            end
        end
        
        % search for local minimum regions (either because of mean or
        % covariance)
        if ( GP.mean.mean_t.local_region_dependency_b || ...
                GP.covariance.kernel_t.parameters_t.local_region_dependency_b || ...
                optimization_problem.algorithm_parameters_t.direct_local_optimum_b)
            
            [ local_min_regions_out_t ] = calcAllMinimaRegions(local_min_regions_t,...
                optimization_problem, 0, '');
            [ local_min_regions_out_t ] = orderMinimaRegions(local_min_regions_out_t);
            
            GP.mean.mean_t.parameters_t.local_min_regions_t = ...
                local_min_regions_out_t;
            
            GP.covariance.kernel_t.parameters_t.local_min_regions_t = ...
                    local_min_regions_out_t;
        else
            local_min_regions_out_t.nSegments = 0;
            GP.covariance.kernel_t.parameters_t.local_min_regions_t = ...
                    local_min_regions_out_t;
            GP.mean.mean_t.parameters_t.local_min_regions_t = ...
                local_min_regions_out_t;
        end
        
        % logging
        optimization_process_information.l_f_history(end+1) = GP.covariance.kernel_t.parameters_t.l_f;
        
        %% get initial sub-optimization starting points (for optimization on surface)
        x0 = lhsdesign(optimization_problem.algorithm_parameters_t.N_opt_restarts_i,...
            optimization_problem.dim_i);
        if n >= optimization_problem.algorithm_parameters_t.N_opt_restarts_i
            % add best data point found so far
            [y_values,ID] = min(optimization_problem.y_t);
            x0(1,:) = optimization_problem.X_t(ID(1),:);
        end
        
        %% save iteration information for logging
        [y_opt, ID_opt] = min(optimization_problem.y_t);
        optimization_process_information.x_opt_history(n,:) = ...
            transformInputSpace(optimization_problem.X_t(ID_opt,:),...
            optimization_problem);
        optimization_process_information.y_opt_history(n,:) = y_opt;        
        
        disp(['Optimization Iteration ',num2str(n)...
            ,'/',num2str(optimization_problem.algorithm_parameters_t.N_i),...
            ', Minimum=',num2str(min(optimization_problem.y_t))]);
        drawnow;
    end
    
    %% save final process information
    optimization_process_information.X = optimization_problem.X_t;
    optimization_process_information.y = optimization_problem.y_t;
end

%% helpers...
function [optimization_problem_reduced, GP_reduced] = removeIDFromData(IDs, GP, optimization_problem)
optimization_problem_reduced = optimization_problem;
GP_reduced = GP;

% remove data
X_reduced_t = [];
y_reduced_t = [];
all_data_X_t = optimization_problem.X_t;
all_data_y_t = optimization_problem.y_t;
for nn = 1:1:optimization_problem.n_data_i
    if isempty( find( IDs == nn))
        X_reduced_t(end + 1,:) = optimization_problem.X_t(nn,:);
        y_reduced_t(end + 1,:) = optimization_problem.y_t(nn,:);
    end
end
optimization_problem_reduced.X_t = X_reduced_t;
optimization_problem_reduced.y_t = y_reduced_t;
optimization_problem_reduced.n_data_i = numel(y_reduced_t);

GP_reduced = doStatisticalParameterEstimation(GP, optimization_problem_reduced);
GP_reduced.covariance.kernel_t.parameters_t.sigmaF_f = ...
    GP_reduced.covariance.kernel_t.parameters_t.sigmaF_f * 0.01;
end

function GP = doStatisticalParameterEstimation(GP, optimization_problem)
    if optimization_problem.algorithm_parameters_t.statistical_parameter_estimation
        GP.mean.mean_t.parameters_t.mu_f = ...
            sum(optimization_problem.y_t)/numel(optimization_problem.y_t);
        GP.covariance.kernel_t.parameters_t.sigmaF_f = sqrt((...
            sum((optimization_problem.y_t - GP.mean.mean_t.parameters_t.mu_f).^2)...
            /(numel(optimization_problem.y_t)-1) ));
    end
end