function [ local_min_regions_out_t ] = calcAllMinimaRegions(local_min_regions_t,...
    optimization_problem, animation_b, animation_path_c)

disp('calcAllMinimaRegions started...');

debug_animation = 0;

local_min_regions_out_t = local_min_regions_t;

% create search grid for input space in case we do not use data points as centers
if local_min_regions_t.search.parameters_t(3) == 0
    grid_ss_t = GRID_interface(zeros(optimization_problem.dim_i,1),...
        ones(optimization_problem.dim_i,1),ones(1,optimization_problem.dim_i)...
        *local_min_regions_t.search.parameters_t(1));
    grid_ss_t = initGridIteration(grid_ss_t);
end

objective_list = [];
end_reached = 0;
if local_min_regions_t.search.parameters_t(3) == 0
    coordinate_values_t = grid_ss_t.count.value_t;
elseif local_min_regions_t.search.parameters_t(3) == 1
    coordinate_values_t = optimization_problem.X_t(1,:);
    current_data_point_i = 1;
end
d = optimization_problem.dim_i;
N_u = 1 + d + (d*(d+1))/2;
while end_reached == 0
    for knn = N_u:1:min(N_u*2,optimization_problem.n_data_i-1)
        % find k nearest neighbours
        IDs = knnsearch(optimization_problem.X_t,coordinate_values_t,...
            'k',knn);

        % calc radius
        knn_radius = norm(coordinate_values_t - ...
            optimization_problem.X_t(IDs(end),:));

        % do regression with k-NN
        if strcmp(local_min_regions_t.search.type_c,'quadReg')
            [beta_t] = quadReg(optimization_problem.X_t(IDs,:),...
                optimization_problem.y_t(IDs));
            if debug_animation
                hold off;
                if optimization_problem.dim_i == 1
                    scatter(optimization_problem.X_t(:,1),...
                        optimization_problem.y_t(:),'g');
                end
                if optimization_problem.dim_i == 2
                    scatter3(optimization_problem.X_t(:,1),...
                        optimization_problem.X_t(:,2),...
                        optimization_problem.y_t(:),'g');
                end
                hold all;
                plotRegression(optimization_problem.X_t(IDs,:),...
                    optimization_problem.y_t(IDs),beta_t,Sigma_beta_t,...
                    optimization_problem.sigmaS_f);
                drawnow;
            end
        end

        % check for positive definite Hessian with local optimum in region
        positive_definite = checkPositiveDefinite(beta_t,optimization_problem.dim_i);
        if ~positive_definite
            continue;
        end

        % check local minimum inside region or not
        [x_opt_local,y_opt_local] = calcLocalMinimum(beta_t,...
            optimization_problem.dim_i);
        if norm(x_opt_local-coordinate_values_t) > knn_radius
            continue;
        end

        % check local minimum inside search space
        bConstraints_local_minimum = 1;
        for i = 1:1:optimization_problem.dim_i
            if 0 > x_opt_local(i)
                bConstraints_local_minimum = 0;
            end
            if 1 < x_opt_local(i)
                bConstraints_local_minimum = 0;
            end
        end

        % check if local minimum beats minimum y value so far
        if ~(y_opt_local < min(optimization_problem.y_t) )
            bConstraints_local_minimum = 0;
        end

        % check nearest point from local minimum (for convergence)
         % find 1 NN
        ID_one =  knnsearch(optimization_problem.X_t,x_opt_local,'k',1);
         % calc radius
        knn_radius_one = norm(x_opt_local - ...
            optimization_problem.X_t(ID_one,:));
        if knn_radius_one < 0.000000001
            bConstraints_local_minimum = 0;
        end

        % check if local minimum is somehow isolated
         % find k+1 nearest neighbours
        IDs_plusOne = knnsearch(optimization_problem.X_t,coordinate_values_t,...
            'k',knn+1);
         % calc radius
        knn_radius_plusOne = norm(coordinate_values_t - ...
            optimization_problem.X_t(IDs_plusOne(end),:));
        if knn_radius_plusOne - knn_radius < .05
            bConstraints_local_minimum = 0;
        end

        if bConstraints_local_minimum == 0
            continue
        end

        % show found local minimum
        if debug_animation
            if optimization_problem.dim_i == 1
                hold all
                scatter(x_opt_local(1),y_opt_local,'m');
            end
            if optimization_problem.dim_i == 2
                hold all
                scatter3(x_opt_local(1),x_opt_local(2),y_opt_local,'m');
            end
            disp(['x_opt_local=',num2str(x_opt_local)]);
            disp('[debug_animation] press any key..');
            pause;
        end

        % save region
        local_min_regions_out_t.nSegments = local_min_regions_out_t.nSegments + 1;
        % location of search start
        local_min_regions_out_t.segments_t(local_min_regions_out_t.nSegments).center ...
            = coordinate_values_t;
        % optimum
        local_min_regions_out_t.segments_t(local_min_regions_out_t.nSegments).local_optimum ...
            = x_opt_local;
        % radius
        local_min_regions_out_t.segments_t(local_min_regions_out_t.nSegments).parameters(1)...
            = knn_radius;
        % knn
        local_min_regions_out_t.segments_t(local_min_regions_out_t.nSegments).parameters(2)...
            = knn;
        % mean regression coefficients
        local_min_regions_out_t.segments_t(local_min_regions_out_t.nSegments).beta...
            = beta_t;
        % objective
        local_min_regions_out_t.segments_t(local_min_regions_out_t.nSegments).objective...
            = y_opt_local;
        % IDs of local data
        local_min_regions_out_t.segments_t(local_min_regions_out_t.nSegments).X_IDs...
            = IDs;
    end
    % next grid point in input space (either full grid or data points)
    if local_min_regions_t.search.parameters_t(3) == 0
        [ grid_ss_t,coordinate_values_t, end_reached ] = gridCount( grid_ss_t );
    elseif local_min_regions_t.search.parameters_t(3) == 1
        if current_data_point_i == numel(optimization_problem.X_t(:,1))
            end_reached = 1;
        else
            current_data_point_i = current_data_point_i + 1;
            coordinate_values_t = optimization_problem.X_t(current_data_point_i,:);
        end
    end

end

disp(['calcAllMinimaRegions finished, found ',...
    num2str(local_min_regions_out_t.nSegments),' regions']);
