function visualizeAllMinimaRegions(local_min_regions_t,...
    optimization_problem,num_regions_to_plot)

if optimization_problem.dim_i == 2
    [X,Y] = meshgrid(linspace(0,1,70),...
        linspace(0,1,30));

end
if optimization_problem.dim_i == 1
    X = linspace(0,1,100);
end
Z = {};

% interate over every segment
for n = 1:1:min(local_min_regions_t.nSegments,num_regions_to_plot)
    % query data
    IDs = knnsearch(optimization_problem.X_t,...
        local_min_regions_t.segments_t(n).center,'k',...
        local_min_regions_t.segments_t(n).parameters(2));
    % do regression
    if strcmp(local_min_regions_t.search.type_c,'quadReg')
        [beta_t] = quadReg(optimization_problem.X_t(IDs,:),...
            optimization_problem.y_t(IDs));
    end
    % do prediction for area
    if optimization_problem.dim_i == 2
        [Z_local ] = quadPred( [X(:),Y(:)] ,beta_t );
    end
    if optimization_problem.dim_i == 1
        [Z_local ] = quadPred( [X(:)] ,beta_t );
    end
    Z{n} = reshape(Z_local,size(X));
end

% generate plot data
X_out = [];
Y_out = [];
Z_out = [];
for n = 1:1:numel(X(:))
    % check in which segment point lies
    for nn = 1:1:min(local_min_regions_t.nSegments,num_regions_to_plot)
        if optimization_problem.dim_i == 2
            if norm([X(n),Y(n)] - local_min_regions_t.segments_t(nn).center)...
                    < local_min_regions_t.segments_t(nn).parameters(1)
                X_out = [X_out X(n)];
                Y_out = [Y_out Y(n)];
                Z_out = [Z_out Z{nn}(n)];
            end
        end
        if optimization_problem.dim_i == 1
            if norm([X(n)] - local_min_regions_t.segments_t(nn).center)...
                    < local_min_regions_t.segments_t(nn).parameters(1)
                X_out = [X_out X(n)];
                Z_out = [Z_out Z{nn}(n)];
            end
        end
    end
end

color_local_region = 'r';

if optimization_problem.dim_i == 2
    scatter3(X_out,Y_out,Z_out,color_local_region);
end
if optimization_problem.dim_i == 1
    scatter(X_out,Z_out,color_local_region);
end
hold on;
if optimization_problem.dim_i == 2
    scatter3(optimization_problem.X_t(:,1),optimization_problem.X_t(:,2),...
        optimization_problem.y_t(:),'b');
    scatter3(optimization_problem.X_t(local_min_regions_t.segments_t(1).X_IDs,1),optimization_problem.X_t(local_min_regions_t.segments_t(1).X_IDs,2),...
        optimization_problem.y_t(local_min_regions_t.segments_t(1).X_IDs),'g');

end
if optimization_problem.dim_i == 1
    scatter(optimization_problem.X_t(:,1),...
        optimization_problem.y_t(:),'b');
end

end
