if(~isdeployed)
  cd(fileparts(which(mfilename)));
end

close all;

%% add relevant paths
addpath(genpath('GP'));
addpath(genpath('localMinimaDetector'));
addpath(genpath('quadRegression'));
addpath(genpath('benchmarkProblems'));
addpath(genpath('searchPolicies'));
addpath(genpath('tools'));
addpath(genpath('shadedErrorBar'));
addpath(genpath('configurations'));
addpath(genpath('results'));
addpath(genpath('lengthscaleController'));
addpath(genpath('kimOptimization'));
addpath(genpath('referenceAlgorithms'));
addpath(genpath('paper'));

%% specify optimization problem (see benchmarkProblems folder)
optimization_problem = createOptimizationProblem_rosenbrock2D;
%optimization_problem = createOptimizationProblem_branin2D;
%optimization_problem = createOptimizationProblem_hole2D; % (exponential 2D)
%optimization_problem = createOptimizationProblem_hole3D; % (exponential 3D)
%optimization_problem = createOptimizationProblem_hole4D; % (exponential 4D)
%optimization_problem = createOptimizationProblem_hole5D; % (exponential 5D)
%optimization_problem = createOptimizationProblem_hartmann3D;
%optimization_problem = createOptimizationProblem_hartmann6D;

%% run algorithm configuration
run('alphaPoly_cfg'); % configuration used for paper results

% number of iterations (caution: in case of quadratic like functions as
% rosenbrock >15 iterations can lead to numerical issues)
optimization_problem.algorithm_parameters_t.N_i = 15;

%% run gbo (Gaussian Bayesian Optimization) with alpha-ratio cool down and MGL kernel with animation
result = gbo(optimization_problem);

%% plot simple regret
y_opt = optimization_problem.fun_func( optimization_problem.x_opt_ground_truth_t);
plot(log10(result.y_opt_history-y_opt));
xlabel('iteration');
ylabel('log10 regret');
grid on;
